@@ -9,11 +9,11 @@
 #ifndef _RH_WAIVED_H
 #define _RH_WAIVED_H
 
-enum rh_waived_feat {
-	/* RH_WAIVED_FEAT_ITEMS must always be the last item in the enum */
-	RH_WAIVED_FEAT_ITEMS,
+enum rh_waived_items {
+	/* RH_WAIVED_ITEMS must always be the last item in the enum */
+	RH_WAIVED_ITEMS,
 };
 
-bool is_rh_waived(enum rh_waived_feat feat);
+bool is_rh_waived(enum rh_waived_items feat);
 
 #endif /* _RH_WAIVED_H */
